@@ -19,13 +19,12 @@
 #
 
 from __future__ import absolute_import, division, print_function, unicode_literals
+import re
+
 from guessit.containers import PropertiesContainer
 from guessit.matcher import GuessFinder
-
 from guessit.plugins.transformers import Transformer
 
-import re
-
 
 class ExpectedSeries(Transformer):
     def __init__(self):
@@ -38,7 +37,8 @@ class ExpectedSeries(Transformer):
     def should_process(self, mtree, options=None):
         return options and options.get('expected_series')
 
-    def expected_series(self, string, node=None, options=None):
+    @staticmethod
+    def expected_series(string, node=None, options=None):
         container = PropertiesContainer(enhance=True, canonical_from_pattern=False)
 
         for expected_serie in options.get('expected_series'):
